import pytest

from infection_monkey.exploit.powershell_utils.credentials import Credentials, SecretType
from infection_monkey.exploit.powershell_utils.powershell_client import format_password


def test_format_cached_credentials():
    expected = None
    creds = Credentials("test_user", expected, SecretType.CACHED)

    actual = format_password(creds)

    assert expected == actual


def test_format_password():
    expected = "test_password"
    creds = Credentials("test_user", expected, SecretType.PASSWORD)

    actual = format_password(creds)

    assert expected == actual


def test_format_lm_hash():
    lm_hash = "c080132b6f2a0c4e5d1029cc06f48a92"
    expected = f"{lm_hash}:00000000000000000000000000000000"

    creds = Credentials("test_user", lm_hash, SecretType.LM_HASH)

    actual = format_password(creds)

    assert expected == actual


def test_format_nt_hash():
    nt_hash = "c080132b6f2a0c4e5d1029cc06f48a92"
    expected = f"00000000000000000000000000000000:{nt_hash}"

    creds = Credentials("test_user", nt_hash, SecretType.NT_HASH)

    actual = format_password(creds)

    assert expected == actual


def test_invalid_secret_type():

    creds = Credentials("test_user", "secret", "Bogus_Secret")

    with pytest.raises(ValueError):
        format_password(creds)
